@@ -25,12 +25,12 @@ from asyncdb.utils import *
 
 class redisPool(BasePool):
     _dsn = "redis://{host}:{port}/{db}"
-    _max_queries = 10
+    _max_queries = 300
     _pool = None
     _connection = None
     _encoding = "utf-8"
 
-    def __init__(self, dsn="", loop=None, params={}):
+    def __init__(self, dsn="", loop=None, params={}, **kwargs):
         super(redisPool, self).__init__(dsn=dsn, loop=loop, params=params)
         self._pool = None
         try:
@@ -38,6 +38,8 @@ class redisPool(BasePool):
                 self._encoding = params["encoding"]
         except KeyError:
             pass
+        if 'max_queries' in kwargs:
+            self._max_queries = kwargs["max_queries"]
 
     def get_event_loop(self):
         return self._loop
@@ -45,6 +47,9 @@ class redisPool(BasePool):
     def get_connection(self):
         return self._connection
 
+    def is_closed(self):
+        return self._pool.closed
+
     """
     Context magic Methods
     """
@@ -56,18 +61,20 @@ class redisPool(BasePool):
         self._loop.run_until_complete(self.release())
 
     # Create a redis connection pool
-    async def connect(self):
+    async def connect(self, **kwargs):
         """
         __init async db initialization
         """
-        # logger.info("AsyncRedis: Connecting to {}".format(self._dsn))
+        self.logger.debug("Redis Pool: Connecting to {}".format(self._dsn))
         try:
             self._pool = await aioredis.create_pool(
                 self._dsn,
-                minsize=3,
+                minsize=5,
                 maxsize=self._max_queries,
                 loop=self._loop,
                 encoding=self._encoding,
+                create_connection_timeout=300,
+                **kwargs
             )
         except (aioredis.ProtocolError) as err:
             raise ConnectionTimeout(
@@ -110,7 +117,7 @@ class redisPool(BasePool):
             raise ProviderError("Unknown Error: {}".format(str(err)))
             return False
         if self._connection:
-            db = redis(pool=self)
+            db = redis(connection=self._connection, pool=self)
         return db
 
     async def release(self, connection=None):
@@ -120,27 +127,41 @@ class redisPool(BasePool):
         if not connection:
             conn = self._connection
         else:
-            conn = connection
+            if isinstance(connection, redis):
+                conn = connection.engine()
+            else:
+                conn = connection
         try:
             if conn:
                 self._pool.release(conn)
         except Exception as err:
             raise ProviderError("Release Error: {}".format(str(err)))
 
-    async def close(self, timeout=10):
+    async def close(self, timeout=5):
         """
         Close Pool
         """
         try:
-            if self._pool:
+
+            if self._pool and self._pool.closed is False:
                 self._pool.close()
-                await self._pool.wait_closed()
+                # also clear:
+                await self._pool.clear()
+                #await asyncio.sleep(1)
+                close = asyncio.create_task(self._pool.wait_closed())
+                try:
+                    await asyncio.wait_for(
+                        close,
+                        timeout=timeout
+                    )
+                except asyncio.exceptions.TimeoutError as err:
+                    pass
         except (
             aioredis.errors.PoolClosedError, aioredis.ConnectionClosedError
         ) as err:
             raise ProviderError("Connection close Error: {}".format(str(err)))
         except Exception as err:
-            print("Pool Closing Error: {}".format(str(err)))
+            logging.exception("Pool Closing Error: {}".format(str(err)))
             return False
 
     async def execute(self, sentence, *args, **kwargs):
@@ -179,11 +200,11 @@ class redis(BaseProvider):
     _loop = None
     _encoding = "utf-8"
 
-    def __init__(self, dsn="", loop=None, pool=None, params={}):
+    def __init__(self, dsn="", connection=None, loop=None, pool=None, params={}):
         super(redis, self).__init__(dsn=dsn, loop=loop, params=params)
         if pool:
             self._pool = pool
-            self._connection = aioredis.Redis(pool.get_connection())
+            self._connection = aioredis.Redis(connection)
             self._connected = True
             self._initialized_on = time.time()
         try:
@@ -195,6 +216,14 @@ class redis(BaseProvider):
     """
     Context magic Methods
     """
+    async def __aenter__(self) -> "BaseProvider":
+        if not self._connection:
+            await self.connection()
+        return self
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
+        # clean up anything you need to clean up
+        return await self.release()
 
     def __enter__(self):
         return self
@@ -218,14 +247,18 @@ class redis(BaseProvider):
         return self._connection
 
     # Create a redis pool
-    async def connection(self):
+    async def connection(self, **kwargs):
         """
         __init async redis initialization
         """
-        # logger.info("AsyncRedis: Connecting to {}".format(self._dsn))
+        self._logger.info("AsyncRedis: Connecting to {}".format(self._dsn))
         try:
             connection = await aioredis.create_connection(
-                self._dsn, loop=self._loop, encoding=self._encoding
+                self._dsn,
+                loop=self._loop,
+                encoding=self._encoding,
+                timeout=self._timeout,
+                **kwargs
             )
             self._connection = aioredis.Redis(connection)
         except (aioredis.ProtocolError, aioredis.AuthError) as err:
@@ -251,15 +284,30 @@ class redis(BaseProvider):
         Release a connection and return into pool
         """
         if self._pool:
-            await self._pool.release(connection=self._connection)
+            await self._pool.release(
+                connection=self._connection.connection
+            )
+        try:
+            await self._connection.quit()
+        except aioredis.errors.ConnectionClosedError as e:
+            self._logger.error(f'Error releasing the Connection {e!s}')
+
+    def is_closed(self):
+        if not self._connection:
+            return True
+        else:
+            return self._connection.closed
+
+    async def ping(self, msg: str = ''):
+        await self._connection.ping(msg)
 
     async def close(self):
-        if self._connection:
-            try:
-                self._connection.close()
-                await self._connection.wait_closed()
-            finally:
-                self._connection = None
+        try:
+            self._connection.close()
+            await asyncio.shield(self._connection.wait_closed())
+        finally:
+            self._connection = None
+            self._connected = False
 
     async def execute(self, sentence, *args):
         if self._connection:
@@ -269,7 +317,7 @@ class redis(BaseProvider):
                 return result
             except (
                 aioredis.errors.PoolClosedError,
-                aioredis.ConnectionClosedError,
+                aioredis.errors.ConnectionClosedError,
             ) as err:
                 raise ProviderError("Connection Error: {}".format(str(err)))
 
@@ -322,7 +370,9 @@ class redis(BaseProvider):
             raise Exception
             return False
 
-    def exists(self, key, *keys):
+    async def exists(self, key, *keys):
+        if not self._connection:
+            await self.connection()
         try:
             return self._connection.exists(key, *keys)
         except (aioredis.RedisError, aioredis.ProtocolError) as err:
