@@ -560,7 +560,7 @@ class SQLProvider(BaseProvider):
         fields = model.columns(model)
         for name, field in fields.items():
             val = getattr(model, field.name)
-            print(name, field, val)
+            # print(name, field, val)
             column = field.name
             datatype = field.type
             value = Entity.toSQL(val, datatype)
@@ -690,11 +690,49 @@ class SQLProvider(BaseProvider):
             result = await self._connection.execute(sql)
             if result:
                 sql = f'SELECT {columns} FROM {table} {condition}'
+                print('UPDATE ', sql)
                 return await self._connection.fetch(sql)
         except Exception as err:
             print(traceback.format_exc())
             raise Exception('Error on Insert over table {}: {}'.format(model.Meta.name, err))
 
+    async def delete_rows(self, model: Model, conditions: dict, **kwargs):
+        """
+        Deleting some records and returned.
+        """
+        if not self._connection:
+            await self.connection()
+        table = f'{model.Meta.schema}.{model.Meta.name}'
+        source = []
+        pk = {}
+        cols = []
+        fields = model.columns(model)
+        for name, field in fields.items():
+            column = field.name
+            datatype = field.type
+            cols.append(column)
+            if column in kwargs:
+                value = Entity.toSQL(kwargs[column], datatype)
+                source.append('{} = {}'.format(
+                    column,
+                    Entity.escapeLiteral(
+                        value, datatype
+                        )
+                    )
+                )
+        set_fields = ', '.join(source)
+        condition = self._where(fields, **conditions)
+        columns = ', '.join(cols)
+        sql = f'DELETE FROM {table} {condition}'
+        print(sql)
+        try:
+            result = await self._connection.execute(sql)
+            if result:
+                return result
+        except Exception as err:
+            print(traceback.format_exc())
+            raise Exception('Error on Deleting table {}: {}'.format(model.Meta.name, err))
+
     async def create_rows(self, model: Model, rows: list):
         """
         Create all records based on rows and returned
@@ -703,45 +741,56 @@ class SQLProvider(BaseProvider):
             await self.connection()
         table = f'{model.Meta.schema}.{model.Meta.name}'
         fields = model.columns(model)
-        n = len(fields)
-        columns = ', '.join(fields.keys())
-        values = ','.join(['${}'.format(a) for a in range(1, n+1)])
-        primary = 'RETURNING *'
-        insert = f'INSERT INTO {table} ({columns}) VALUES ({values}) {primary}'
-        print(insert)
-        logging.debug(f'INSERT: {insert}')
-        try:
-            stmt = await self._connection.prepare(insert)
-        except Exception as err:
-            print(traceback.format_exc())
-            raise Exception(
-                'Exception creating Prepared Sentence {}: {}'.format(
-                    model.Meta.name, err)
-                )
         results = []
+        stmt = None
         for row in rows:
             source = []
             pk = []
+            cols = []
             for col, field in fields.items():
                 if col not in row:
                     # field doesnt exists
                     default = field.default
-                    if default:
+                    if default is not None:
                         if callable(default):
                             source.append(default())
                         else:
                             source.append(default)
+                        cols.append(col)
                     else:
-                        if field.required is True or field.primary_key is True:
+                        # val = getattr(model, col)
+                        # if val is not None:
+                        #     source.append(val)
+                        # elif field.required is True or field.primary_key is True:
+                        if field.required is True:
                             raise StatementError(
                                 f'Missing Required Field: {col}'
                             )
-                        else:
-                            source.append(None)
                 else:
-                    source.append(row[col])
+                    try:
+                        val = row[col]
+                        source.append(val)
+                        cols.append(col)
+                    except (KeyError, TypeError):
+                        continue
                 if field.primary_key is True:
                     pk.append(col)
+            if not stmt:
+                columns = ', '.join(cols)
+                n = len(cols)
+                values = ','.join(['${}'.format(a) for a in range(1, n+1)])
+                primary = 'RETURNING *'
+                insert = f'INSERT INTO {table} ({columns}) VALUES ({values}) {primary}'
+                print(insert)
+                logging.debug(f'INSERT: {insert}')
+                try:
+                    stmt = await self._connection.prepare(insert)
+                except Exception as err:
+                    print(traceback.format_exc())
+                    raise Exception(
+                        'Exception creating Prepared Sentence {}: {}'.format(
+                            model.Meta.name, err)
+                        )
             try:
                 result = await stmt.fetchrow(*source, timeout=2)
                 logging.debug(stmt.get_statusmsg())
